#!/usr/bin/env python3
from sysrepo_api import sysrepo_api3 as api
import sdnotify
import random
import signal
import time
import paramiko
import subprocess
import mysql.connector
import json

SSH_PORT_FILE = '/config/ssh_port'


class SysrepoMonitoring(api.SysrepoAPI):

    def __init__(self,
                 app_name="monitoring",
                 log_file=None,
                 log_level=None,
                 console_log=False,
                 run_as_subscriber=False,
                 priority=0
                 ):
        api.SysrepoAPI.__init__(
            self,
            app_name=app_name,
            module_name="ucpe-pm-statistics",
            log_file=log_file,
            log_level=log_level,
            console_log=console_log,
            run_as_subscriber=run_as_subscriber,
            priority=priority
        )

    def add_subscribers(self):
        # Alive data getter subscriber
        if len(self.status_callbacks) > 0:
            for i in self.status_callbacks:
                self.subscriber.subscribe_oper_data_request(self.module_name, i['path'], i['callback'])
        elif self.status_namespace:
            self.subscriber.subscribe_oper_data_request(self.module_name, self.status_namespace, self.status_get_cb)

    def run_subscriber(self):
        # Handle proper termination in case of service
        self.logger.info("Starting the subscriber service...")
        while True:
            try:
                signal.signal(signal.SIGINT, self._terminate_signal)

                try:
                    # Lock the other subscribers
                    self.session_start()
                    self.subscriber = self.session
                    self.add_rpc_cb()
                    self.add_subscribers()
                    ntf = sdnotify.SystemdNotifier()
                    ntf.notify("READY=1")
                except KeyError as k_err_ex:
                    self.logger.error("No such key {}".format(k_err_ex))
                    pass
                except Exception as ex:
                    raise Exception(ex)

                signal.sigwait({signal.SIGINT, signal.SIGTERM})
            except KeyError as k_err_ex:
                self.logger.error("No such key {}".format(k_err_ex))
                pass
            except Exception as ex:
                self.logger.error(ex)
                self.session_stop()

            milliseconds = random.randint(500, 3000)
            self.logger.error(
                "Subscriber exited. Waiting {} milliseconds for the next start ...".format(milliseconds))
            time.sleep(milliseconds / 1000)

    def _get_ssh_port(self):
        try:
            with open(SSH_PORT_FILE, 'r') as file:
                content = file.read().strip()
                ssh_port = int(content) if content else 22222

        except FileNotFoundError:
            self.logger.error(f"Error: The file {SSH_PORT_FILE} was not found. Using default port 22222.")
            ssh_port = 22222
        except Exception as e:
            self.logger.error(f"Error occurred: {e} while getting the ssh-port.")
            ssh_port = 22222

        return ssh_port

    def exec_cli_cmd(self, command):
        try:
            passlines = subprocess.check_output(["sudo", "cat", "/root/admin-openrc.env"], stderr=subprocess.STDOUT)
        except subprocess.CalledProcessError as e:
            # LOG.error("fail exception CalledProcessError")
            return "{}".format(e)
        except OSError as e:
            # LOG.error("fail exception OSError")
            return "{}".format(e)

        pass_list = passlines.decode("utf-8").split('\n')
        password = pass_list[4][12:]
        sshclient = paramiko.SSHClient()
        sshclient.load_system_host_keys()
        ssh_port = self._get_ssh_port()
        sshclient.set_missing_host_key_policy(paramiko.client.AutoAddPolicy())
        sshclient.connect('127.0.0.1', port=ssh_port, username='admin', password=password, look_for_keys=False)
        stdin, stdout, stderr = sshclient.exec_command(command, bufsize=0)
        stdout = stdout.readlines()
        stderr = stderr.readlines()
        sshclient.close()
        output = ''
        for line in stdout:
            if line.startswith('Welcome admin it is'):
                continue

            output = output + line
        if output != '':
            return "{}".format(output)
        for line in stderr:
            if line.startswith('Welcome admin it is'):
                continue

            output = output + line
        if output != '':
            return "{}".format(output)
        return "OK"

    def ucpe_rpc_cb(self, xpath, input_params, event, private_data):
        output = "OK"
        try:
            if (input_params == None):
                return None
            output = {"result": "%s" % self.exec_cli_cmd(input_params["command"])}
        except Exception as e:
            print(e)
            return None
        return output

    def get_last_interval_id(self, cursor):
        sql = "SELECT * FROM state_data"
        cursor.execute(sql)
        records = cursor.fetchall()
        for row in records:
            interval = row[1]
            return interval
        return 0

    def ucpe_pm_status_cb(self, xpath, private_data):
        data = {"interval": '', "data": []}
        reply = {"ucpe-pm-statistics:pm-statistics": {}}
        payload = []
        interval_id = 0
        interval_id2 = 0
        last_interval_id = 0
        curr_interval_flag = 0

        try:
            mydb = mysql.connector.connect(host='localhost', user="root", passwd="root", database="collectd")
            cursor = mydb.cursor()

            if xpath.find('current-interval') >= 0:
                # last interval ID
                last_interval_id = self.get_last_interval_id(cursor)
                reply["ucpe-pm-statistics:pm-statistics"]["current-interval"] = "{}".format(last_interval_id)
                mydb.close()
                return reply

            if xpath.find('interval[id=\'') >= 0:
                # get interval/s
                interval_id = int(
                    xpath[xpath.find('interval[id=\'') + len('interval[id=\''):xpath.rfind('\'')])
                #                if interval_id.find('-') > 0:
                #                    interval_id, interval_id2 = map(int, interval_id.split('-'))
                #                    if interval_id > interval_id2: interval_id, interval_id2 = interval_id2, interval_id
                #                else:
                #                    interval_id = int(interval_id)

                #                if interval_id2 > 0:
                #                    sql = "SELECT * FROM pmstats WHERE interval_id BETWEEN %s AND %s"%(interval_id,interval_id2)
                #                elif interval_id == 0:
                #                    sql = "SELECT * FROM pmstats WHERE interval_id = %s"%self.get_last_interval_id(cursor)
                #                elif interval_id > 0:
                #                    sql = "SELECT * FROM pmstats WHERE interval_id = %s"%interval_id
                sql = "SELECT * FROM pmstats WHERE interval_id = %s" % interval_id

            elif (xpath.find('/ucpe-pm-statistics:pm-statistics//') >= 0):
                # All
                last_interval_id = self.get_last_interval_id(cursor)
                sql = "SELECT * FROM pmstats WHERE time > unix_timestamp()-(24*60*60)"
            elif (xpath.find('/ucpe-pm-statistics:pm-statistics/interval//') >= 0):
                sql = "SELECT * FROM pmstats WHERE time > unix_timestamp()-(24*60*60)"
            else:
                # Not a valid request
                mydb.close()
                return reply

            cursor.execute(sql)
            records = cursor.fetchall()
            curr_interval = -1
            for row in records:
                if curr_interval == -1:
                    curr_interval = row[0]
                if curr_interval == row[0]:
                    data['interval'] = row[0]
                    data['data'].append(json.loads(row[2]))
                else:
                    payload.append(data)
                    curr_interval = row[0]
                    data = {"interval": '', "data": []}
                    data['interval'] = row[0]
                    data['data'].append(json.loads(row[2]))

            payload.append(data)
            mydb.close()
            if last_interval_id != 0:
                curr_interval_flag = 1

            if curr_interval_flag != 0:
                reply["ucpe-pm-statistics:pm-statistics"]["current-interval"] = "{}".format(last_interval_id)
            reply["ucpe-pm-statistics:pm-statistics"]["interval"] = []
            for i in range(0, len(payload)):
                reply["ucpe-pm-statistics:pm-statistics"]["interval"].append({ "id": "%s" % payload[i]['interval'],
                                                                               "data": "%s" % payload[i]['data']
                                                                             })

        except OSError as e:
            return None
        except Exception as e:
            self.logger.error(str(e))
            return None

        return reply
